import { get, set, del, update, createStore } from 'idb-keyval'

export class Store {
    constructor(data) {
        this.name = data.name
        this.store = data.store

        // 检查过期的数据
        this.checkExpired()
    }

    async checkExpired() {
        const keys = await this.store.keys()
        const now = Date.now()
        for (const key of keys) {
            const item = await get(key, this.store)
            if (item && item.expiresAt && item.expiresAt < now) {
                await this.store.delete(key)
            }
        }
    }

    async create(data) {
        const id = window.crypto.randomUUID()
        const createdAt = Date.now()
        const updatedAt = createdAt
        const lastUsedAt = createdAt
        const item = { id, createdAt, updatedAt, lastUsedAt, ...data }
        await set(id, item, this.store)
        return item
    }

    async delete(id) {
        return await this.store.delete(id)
    }

    async query(query) {
    }

    async get(id) {
        return await get(id, this.store)
    }

    async update(id, data) {
        const item = await this.get(id)
        if (!item) {
            return null
        }
        const updatedAt = Date.now()
        const lastUsedAt = updatedAt
        const item2 = { ...item, updatedAt, lastUsedAt, ...data }
        await set(id, item2, this.store)
        return item2
    }
}

export const musicStore = new Store({
    name: '音乐(资源类型)',
    store: createStore('db-music', 'store-music')
})

export const chatStore = new Store({
    name: '聊天(资源类型)',
    store: createStore('db-chat', 'store-chat')
})

export const userStore = new Store({
    name: '用户(资源类型)',
    store: createStore('db-user', 'store-user')
})
